import { IDBExecuter } from '../interfaces/dbexecuter.adapter';
import { IGraphEntity } from './neoEntities/graph.entity';
import { ParamsHolder } from './paramsHolder';
import * as Neo4JDriver from 'neo4j-driver';
export declare abstract class DBAction {
    protected LINE_BREAK: string;
    protected TAB_CHAR: string;
    protected RETURN_KEYWORD: string;
    protected returnBuilderCallBack: (entities: IGraphEntity[], params: ParamsHolder) => string;
    private paramsHolder;
    private isFunctionOverrided;
    protected entities: IGraphEntity[];
    protected executerAdapter: IDBExecuter;
    constructor(entities: IGraphEntity[], adapter?: IDBExecuter);
    /**
     * Return a params Object that you can inject to a Neo4J Database Driver, or to export it as JSON
     * @returns {Records<string, any>}
     */
    getParamsForDatabaseUse(): any;
    /**
     * Converts the entities and returns a minified version of the cypher query in string form.
     * @returns {string}
     */
    getQuery(): string;
    /**
     * Converts the entities and return a prettified version of the cypher query in string form.
     * @returns
     */
    getPrettyQuery(): string;
    /**
     * Set new entities to the DBAction this will also auto generate a new set of params to use.
     * @param {IGraphEntity[]} entities - a set of entities used to build the cypher query
     * @returns
     */
    setEntities(entities: IGraphEntity[]): DBAction;
    /**
     * Override the "return" cypher generator function, you can set a new string cypher function to inject
     * custom behaviour at the return part of your query
     * @example
     * // this will return a cypher like:
     * // match(n) return 'hello world';
     * action.overrideReturnAction( () => '\'hello world\'' ).getQuery();
     * @param callBack - This callback will be executed when Neo4TS generates the return part of the cypher query
     * @returns
     */
    overrideReturnAction(callBack: (entities: IGraphEntity[], params: ParamsHolder) => string): DBAction;
    /**
     * Returns the param Holder which contains the parameters that will be used by this aciton.
     * @returns
     */
    getParamHolder(): ParamsHolder;
    /**
     * It will execute the action into the Data Base, using Neo4J Driver to connect and execute the cypher, using the entities assigned to this action
     * After executing the action it will try to parse the result to a more friendly JSON form.
     * @returns
     */
    execute(): Promise<any>;
    /**
     * It will execute the action into the Data Base, using Neo4J Driver to connect and execute the cypher, using the entities assigned to this action
     * It will return the raw response from Neo4J Driver
     * @returns
     */
    abstract executeRaw(): Promise<Neo4JDriver.QueryResult>;
    protected abstract buildQueryBody(params: ParamsHolder): string;
    protected buildAfterQueryReturnDecorators(params: ParamsHolder): string;
    protected buildQueryReturn(params: ParamsHolder): string;
    private generateQuery;
    private setParameters;
}
